@@ -40,6 +40,27 @@ DYNAMIC_ATTRIBUTE_MARKERS = [
 # and for sub_(one_)annotation(s).
 DUMMY_CONTAINER = object()
 
+# Names defined on every module/class that should be ignored in most cases.
+TOP_LEVEL_IGNORE = frozenset({
+    "__builtins__",
+    "__doc__",
+    "__file__",
+    "__future__",
+    "__module__",
+    "__name__",
+    "__annotations__",
+    "google_type_annotations",
+})
+CLASS_LEVEL_IGNORE = frozenset({
+    "__builtins__",
+    "__class__",
+    "__module__",
+    "__name__",
+    "__qualname__",
+    "__slots__",
+    "__annotations__",
+})
+
 
 class ConversionError(ValueError):
   pass
@@ -178,6 +199,7 @@ def get_views(variables, node):
 
 
 def get_signatures(func):
+  """Gets the given function's signatures."""
   if func.isinstance_PyTDFunction():
     return [sig.signature for sig in func.signatures]
   elif func.isinstance_InterpreterFunction():
@@ -185,6 +207,8 @@ def get_signatures(func):
   elif func.isinstance_BoundFunction():
     sigs = get_signatures(func.underlying)
     return [sig.drop_first_parameter() for sig in sigs]  # drop "self"
+  elif func.isinstance_ClassMethod() or func.isinstance_StaticMethod():
+    return get_signatures(func.method)
   else:
     raise NotImplementedError(func.__class__.__name__)
 
@@ -202,40 +226,6 @@ def func_name_is_class_init(name):
   return name == "__init__" or name.endswith(".__init__")
 
 
-def has_type_parameters(node, val, seen=None):
-  """Checks if the given object contains any TypeParameters.
-
-  This method differs from the `.formal` attribute on abstract values in that it
-  recurses into container instances to detect type parameters that have
-  (likely incorrectly) been inserted into runtime objects.
-
-  Args:
-    node: The current CFG node.
-    val: The object to check for TypeParameters. Will likely be a
-      SimpleValue or a cfg.Variable with SimpleValues bindings.
-    seen: Optional. A set of already-visited objects, to avoid infinite loops.
-
-  Returns:
-    True if the object contains any TypeParameters, or False otherwise.
-  """
-  if seen is None:
-    seen = set()
-  if val in seen:
-    return False
-  seen.add(val)
-  if isinstance(val, cfg.Variable):
-    return any((has_type_parameters(node, d, seen) for d in val.data))
-  elif val.isinstance_TypeParameter():
-    return True
-  elif val.isinstance_ParameterizedClass() or val.isinstance_Union():
-    return val.formal
-  elif val.isinstance_SimpleValue():
-    return any((has_type_parameters(node, tp, seen)
-                for tp in val.instance_type_parameters.values()))
-  else:
-    return False
-
-
 def equivalent_to(binding, cls):
   """Whether binding.data is equivalent to cls, modulo parameterization."""
   return (binding.data.isinstance_Class() and
@@ -773,8 +763,12 @@ def unwrap_splat(var):
 
 
 def is_callable(value: _BaseValue):
+  """Returns whether 'value' is a callable."""
   if (value.isinstance_Function() or
+      value.isinstance_BoundFunction() or
+      value.isinstance_ClassMethod() or
       value.isinstance_ClassMethodInstance() or
+      value.isinstance_StaticMethod() or
       value.isinstance_StaticMethodInstance()):
     return True
   if not value.cls or not value.cls.isinstance_Class():
