@@ -166,13 +166,9 @@ def add_basic_options(o):
             "precise type-checking. " + temporary))
   o.add_argument(
       "--check-parameter-types", action="store_true",
-      dest="check_parameter_types", default=False,
+      dest="check_parameter_types", default=True,
       help=("Check parameter defaults and assignments against their "
             "annotations. " + temporary))
-  o.add_argument(
-      "--create-pyi-dataclasses", action="store_true",
-      dest="create_pyi_dataclasses", default=False,
-      help="Construct a dataclass from a decorated pyi class. " + temporary)
   o.add_argument(
       "--preserve-union-macros", action="store_true",
       dest="preserve_union_macros", default=False,
