@@ -37,6 +37,7 @@ class Signature:
     annotations: A dictionary of type annotations. (string to type)
     excluded_types: A set of type names that will be ignored when checking the
       count of type parameters.
+    type_params: The set of type parameter names that appear in annotations.
     has_return_annotation: Whether the function has a return annotation.
     has_param_annotations: Whether the function has parameter annotations.
   """
@@ -55,6 +56,10 @@ class Signature:
     if postprocess_annotations:
       for k, annot in self.annotations.items():
         self.annotations[k] = self._postprocess_annotation(k, annot)
+    self.type_params = set()
+    for annot in self.annotations.values():
+      self.type_params.update(
+          p.name for p in annot.vm.annotations_util.get_type_parameters(annot))
 
   @property
   def has_return_annotation(self):
@@ -130,7 +135,7 @@ class Signature:
         name=name,
         param_names=tuple(p.name for p in sig.params if not p.kwonly),
         varargs_name=None if sig.starargs is None else sig.starargs.name,
-        kwonly_params=set(p.name for p in sig.params if p.kwonly),
+        kwonly_params=tuple(p.name for p in sig.params if p.kwonly),
         kwargs_name=None if sig.starstarargs is None else sig.starstarargs.name,
         defaults={p.name: param_to_var(p) for p in sig.params if p.optional},
         annotations={
@@ -149,7 +154,7 @@ class Signature:
         name="<callable>",
         param_names=tuple(sorted(annotations)),
         varargs_name=None,
-        kwonly_params=set(),
+        kwonly_params=(),
         kwargs_name=None,
         defaults={},
         annotations=annotations,
@@ -162,7 +167,7 @@ class Signature:
         name=name,
         param_names=tuple(param_names),
         varargs_name=None,
-        kwonly_params=set(),
+        kwonly_params=(),
         kwargs_name=None,
         defaults={},
         annotations={},
@@ -530,7 +535,11 @@ class DictKeyMissing(Exception, ReturnValueMixin):
 BadCall = collections.namedtuple("_", ["sig", "passed_args", "bad_param"])
 
 
-BadParam = collections.namedtuple("_", ["name", "expected"])
+class BadParam(
+    collections.namedtuple("_", ["name", "expected", "protocol_error"])):
+
+  def __new__(cls, name, expected, protocol_error=None):
+    return super().__new__(cls, name, expected, protocol_error)
 
 
 class InvalidParameters(FailedFunctionCall):
@@ -691,8 +700,8 @@ class PyTDSignature(utils.VirtualMachineWeakrefMixin):
     """Substitute matching args into this signature. Used by PyTDFunction."""
     formal_args, arg_dict = self._map_args(args, view)
     self._fill_in_missing_parameters(node, args, arg_dict)
-    subst, bad_arg = self.vm.matcher.compute_subst(
-        node, formal_args, arg_dict, view, alias_map)
+    subst, bad_arg = self.vm.matcher(node).compute_subst(
+        formal_args, arg_dict, view, alias_map)
     if subst is None:
       if self.signature.has_param(bad_arg.name):
         signature = self.signature
