@@ -187,11 +187,16 @@ def is_dunder(name):
   return name.startswith("__") and name.endswith("__")
 
 
-def instantiate(node, name, typ):
-  # See test_attr.TestAttrib.test_repeated_default - keying on the name prevents
-  # attributes from sharing the same default object.
-  _, instance = typ.vm.init_class(node, typ, extra_key=name)
-  return instance
+def add_member(node, cls, name, typ):
+  if typ.formal:
+    # If typ contains a type parameter, we mark it as empty so that instances
+    # will use __annotations__ to fill in concrete type parameter values.
+    instance = typ.vm.convert.empty.to_variable(node)
+  else:
+    # See test_attr.TestAttrib.test_repeated_default - keying on the name
+    # prevents attributes from sharing the same default object.
+    _, instance = typ.vm.init_class(node, typ, extra_key=name)
+  cls.members[name] = instance
 
 
 def get_class_locals(cls_name, allow_methods, ordering, vm):
@@ -231,3 +236,19 @@ def get_class_locals(cls_name, allow_methods, ordering, vm):
         del out[op.name]
     out[op.name] = local
   return out
+
+
+def make_replace_method(vm, node, cls, *, kwargs_name="kwargs"):
+  """Create a replace() method for a dataclass."""
+  # This is used by several packages that extend dataclass.
+  # The signature is
+  #   def replace(self: T, **kwargs) -> T
+  typevar = abstract.TypeParameter(abstract_utils.T + cls.name, vm, bound=cls)
+  return overlay_utils.make_method(
+      vm=vm,
+      node=node,
+      name="replace",
+      return_type=typevar,
+      self_param=overlay_utils.Param("self", typevar),
+      kwargs=overlay_utils.Param(kwargs_name),
+  )
