@@ -216,13 +216,9 @@ class Cast(abstract.PyTDFunction):
 
   def call(self, node, func, args):
     if args.posargs:
-      annot = self.vm.annotations_util.extract_annotation(
-          node, args.posargs[0], "typing.cast", self.vm.simple_stack())
-      if annot.formal:
-        self.vm.errorlog.invalid_typevar(
-            self.vm.frames, "cannot pass a TypeVar to a function")
-        return node, self.vm.new_unsolvable(node)
-      return self.vm.init_class(node, annot)
+      _, value = self.vm.annotations_util.extract_and_init_annotation(
+          node, "typing.cast", args.posargs[0])
+      return node, value
     return super().call(node, func, args)
 
 
@@ -490,8 +486,6 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
                  "_fields", "_field_defaults", "_field_types",
                  "_make", "_replace", "_asdict", "_source")
 
-  _special = ("__module__", "__name__", "__qualname__", "__annotations__")
-
   def __init__(self, vm):
     typing_ast = vm.loader.import_name("typing")
     pyval = typing_ast.Lookup("typing._NamedTupleClass")
@@ -578,7 +572,7 @@ class NamedTupleClassBuilder(abstract.PyTDClass):
       for key in f_locals:
         if key in self._prohibited:
           self.vm.errorlog.not_writable(self.vm.frames, cls_val, key)
-        if key not in self._special and  key not in fields:
+        if key not in abstract_utils.CLASS_LEVEL_IGNORE and  key not in fields:
           node = self.vm.attribute_handler.set_attribute(
               node, cls_val, key, f_locals[key])
 
